<?php

/**
 * @param $form
 * @param $form_state
 * @param $uid
 * @param null $id_hash
 *
 * @return mixed
 */
function openid_claveunica_complete_login_form($form, &$form_state, $uid, $id_hash) {
  $form['helptext'] = array(
    '#markup' => t('Please fill this form to complete your registration.'),
  );

  $form['mail'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail address'),
    '#maxlength' => EMAIL_MAX_LENGTH,
    '#description' => t('A valid e-mail address. All e-mails from the system will be sent to this address. The e-mail address is not made public and will only be used if you wish to receive a new password or wish to receive certain news or notifications by e-mail.'),
    '#required' => TRUE,
    '#default_value' => '',
  );

  $form['uid'] = array(
    '#type' => 'value',
    '#value' => $uid,
  );
  $form['id_hash'] = array(
    '#type' => 'value',
    '#value' => $id_hash,
  );
  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Complete'),
  );

  return $form;
}

/**
 * @param $form
 * @param $form_state
 */
function openid_claveunica_complete_login_form_validate($form, &$form_state) {
  $uid = $form_state['values']['uid'];
  $id_hash = $form_state['values']['id_hash'];
  if (!openid_claveunica_complete_login_access($uid, $id_hash)) {
    form_set_error('openid_claveunica_validate', t('User ID cannot be identified.'));
    drupal_goto('<front>');
  }

  // Trim whitespace from mail, to prevent confusing 'e-mail not valid'
  // warnings often caused by cutting and pasting.
  $mail = trim($form_state['values']['mail']);
  form_set_value($form['mail'], $mail, $form_state);

  // Validate the e-mail address, and check if it is taken by an existing user.
  if ($error = user_validate_mail($form_state['values']['mail'])) {
    form_set_error('mail', $error);
  }
  elseif ((bool) db_select('users')->fields('users', array('uid'))->condition('uid', $uid, '<>')->condition('mail', db_like($form_state['values']['mail']), 'LIKE')->range(0, 1)->execute()->fetchField()) {
    // Format error message dependent on whether the user is logged in or not.
    if ($GLOBALS['user']->uid) {
      form_set_error('mail', t('The e-mail address %email is already taken.', array('%email' => $form_state['values']['mail'])));
    }
    else {
      form_set_error('mail', t('The e-mail address %email is already registered. <a href="@password">Have you forgotten your password?</a>', array('%email' => $form_state['values']['mail'], '@password' => url('user/password'))));
    }
  }
}

/**
 * @param $form
 * @param $form_state
 *
 * @throws \Exception
 */
function openid_claveunica_complete_login_form_submit($form, &$form_state) {
  global $user;
  $user = user_load((int)$form_state['values']['uid']);
  $fields_value = array(
    'mail' => $form_state['values']['mail'],
  );
  // @TODO: Make this configurable?
  $redirect = '<front>';
  if (!empty($_GET['destination'])) {
    $redirect = $_GET['destination'];
  }
  $form_state['redirect'] = $redirect;
  // Update the user table timestamp noting user has logged in.
  $user->login = REQUEST_TIME;
  db_update('users')
    ->fields(array('login' => $user->login))
    ->condition('uid', $user->uid)
    ->execute();
  watchdog('openid_claveunica', '%name filled missing fields.', array('%name' => $user->name));
  drupal_session_regenerate();
  user_save($user, $fields_value);
  user_module_invoke('login', $edit, $user);
}
